import argparse
import itertools
import threading
import time
import sys


import requests
from termcolor import cprint
import colorama
import enchant

from pyvoc.check_config import read_config_file, check_config_dir
from pyvoc.dmanager import add_word_to_vocab, list_all_groups
from pyvoc.termoutput import revise_vocab, start_quiz, terminal_width
from pyvoc import __version__

import textwrap


colorama.init()
done = False
d = enchant.Dict("en_US")


def parse_dictionary_response(response):
    lexicalEntries = response.json().get("results")[0]["lexicalEntries"]

    parsed_definitions = dict()
    parsed_examples = dict()

    for lexicalEntry in lexicalEntries:
        lexicalCategory = lexicalEntry["lexicalCategory"]["text"]
        try:
            definition = lexicalEntry["entries"][0]["senses"][0]["definitions"][0]
            try:
                example = lexicalEntry["entries"][0]["senses"][0]["examples"][0]["text"]
            except KeyError:
                example = "None"
        except Exception:
            print("No definition found")

        parsed_definitions[lexicalCategory] = definition
        parsed_examples[lexicalCategory] = example

    return parsed_definitions, parsed_examples


def pretty_print_definition(word, parsed_response, examples):
    print("")
    cprint(word + " ", color="cyan", attrs=["bold", "reverse"])
    for key in parsed_response:
        cprint(key + ":", color="green", end=" " * (16 - len(key)))
        width_left = terminal_width - 18
        sentences = textwrap.wrap(parsed_response[key], width=width_left)
        s_count = 1
        for sentence in sentences:
            if s_count == 1:
                print(sentence)
            else:
                print(" " * (17) + sentence)
            s_count += 1
        cprint("example:", color="yellow", end=" " * (16 - len("example")))
        sentences = textwrap.wrap(examples[key], width=width_left)
        s_count = 1
        for sentence in sentences:
            if s_count == 1:
                print(sentence.encode("utf-8").decode("utf-8"))
            else:
                print(" " * (15) + sentence)
            s_count += 1
        print("")


def animate():
    for c in itertools.cycle(["|", "/", "-", "\\"]):
        if done:
            break
        sys.stdout.write("\r " + c)
        sys.stdout.flush()
        time.sleep(0.1)


def stop_loading_animation():
    global done
    done = True


def dictionary(word):
    url = "https://od-api.oxforddictionaries.com/api/v2/entries/en-us/{}?fields=definitions,examples&strictMatch=false".format(
        word.lower()
    )
    check_config_dir()

    app_id, app_key = read_config_file()
    headers = {"app_id": app_id, "app_key": app_key}

    try:
        # checking if the spelling is right using enchant
        if not d.check(word):
            stop_loading_animation()
            cprint("Please check the spelling!\n", color="red")

            # suggesting correct spellings
            possible_correct_spellings = d.suggest(word)
            cprint("suggestions:", color="yellow", attrs=["bold"])
            for word_suggestion in possible_correct_spellings:
                if len(word_suggestion.split(" ")) == 1:
                    cprint(word_suggestion, color="cyan")
        else:
            response = requests.get(url, headers=headers)
            stop_loading_animation()
            print("")

    except ConnectionError:
        print("Unable to connect. Please check your internet connection.")

    if response.status_code == 200:
        parsed_response, examples = parse_dictionary_response(response)
        pretty_print_definition(word, parsed_response, examples)
        return parsed_response
    elif response.status_code == 404:
        cprint("No definition found. Please check the spelling!", color="red")
        exit()
    if response.status_code == 403:
        cprint(
            (
                "You have reached the API limit. This may happen because the api keys are "
                "shared among multiple users. The best thing would be to create your own free personal "
                "api key on https://developer.oxforddictionaries.com and paste them in ~/.pyvoc/pyvoc.config. "
                "Or, you can delete the the config file(~/.pyvoc/pyvoc.config) and run `pyvoc -w hello` to get a new "
                "shared api key"
            ),
            color="red",
        )
        exit()
    elif response.status_code == 500:
        cprint("Internal error. Error occured while processing the data", color="red")
        exit()
    else:
        print("Something went wrong. Check after some time.", color="red")
        exit()


def main():
    parser = argparse.ArgumentParser(
        description="Command line dictionary and vocabulary building tool."
    )

    # add arguments to our CLI
    parser.add_argument(
        "-v", "--version", action="store_true", help="Print version of pyvoc and exit"
    )
    parser.add_argument(
        "-w", dest="word", metavar="<word>", help="Give meaning of WORD"
    )
    parser.add_argument(
        "-a",
        "--add-word",
        action="store_true",
        help="Use to add WORD to vocabulary group",
    )
    parser.add_argument(
        "-g",
        dest="group_num",
        metavar="<group_num>",
        help="Use to specify the vocabulary group no.(1-10) to add the WORD to",
        type=int,
    )
    parser.add_argument(
        "-r",
        dest="revise",
        type=int,
        metavar="<group_num>",
        help="Revise the vocabulary group you mention",
    )
    parser.add_argument(
        "-q",
        dest="quiz",
        metavar="<group_num>",
        type=int,
        help="Start quiz from the vocabulary group you mention",
    )
    parser.add_argument(
        "-n",
        dest="no_of_questions",
        metavar="<no_of_questions>",
        type=int,
        help="Mention the number of questions of quiz.",
    )
    parser.add_argument(
        "-l", "--list", action="store_true", help="Lists all vocabulary groups present"
    )

    args = parser.parse_args()

    # decide the course of action based on the options provided by the user
    # first check the options that does not require connection to the API
    if args.version:
        cprint(__version__, color="white")
        exit()
    elif args.revise:
        revise_vocab(args.revise)
    elif args.quiz:
        cprint("\n\t\tStarting Quiz", color="red", attrs=["bold", "reverse"])
        start_quiz(args.quiz, args.no_of_questions)
    elif args.list:
        list_all_groups()

    # start the loading animation
    t = threading.Thread(target=animate)
    t.start()

    # check the options that require connection to the API
    if args.add_word:
        if not args.word:
            cprint(
                "\nError: Please mention the word to add using option '-w <word>'",
                color="red",
                attrs=["bold"],
            )
            stop_loading_animation()
        else:
            parsed_response = dictionary(args.word)
            add_word_to_vocab(args.word.lower(), parsed_response, args.group_num)
    elif args.word:
        parsed_response = dictionary(args.word)
    else:
        cprint(
            "\nError: No arguments present. Run 'pyvoc --help' to see the available options",
            color="red",
            attrs=["bold"],
        )
        stop_loading_animation()


if __name__ == "__main__":
    main()



import os
import json
import configparser

from termcolor import cprint
import colorama
import requests

colorama.init()

home_dir = os.path.expanduser("~")
config = configparser.ConfigParser()


def config_dir_path():
    return os.path.join(home_dir, ".pyvoc")


config_file_path = os.path.join(config_dir_path(), "pyvoc.config")


def check_config_dir():
    try:
        path = config_dir_path()
        # makes path recursively. returns None if already exist.
        os.makedirs(path, exist_ok=True)
        if not os.path.isfile(os.path.join(path, "group51.json")):
            with open(os.path.join(path, "group51.json"), "w") as f:
                json.dump({}, f)
        if not os.path.isfile(os.path.join(path, "group101.json")):
            response = requests.get(
                "https://raw.githubusercontent.com/makkoncept/definitions/master/group101.json"
            )
            with open(os.path.join(path, "group101.json"), "w") as f:
                json.dump(response.json(), f)
        if not os.path.isfile(os.path.join(path, "group102.json")):
            response = requests.get(
                "https://raw.githubusercontent.com/makkoncept/definitions/master/group102.json"
            )
            with open(os.path.join(path, "group102.json"), "w") as f:
                json.dump(response.json(), f)
        if not os.path.isfile(os.path.join(path, "group103.json")):
            response = requests.get(
                "https://raw.githubusercontent.com/makkoncept/definitions/master/group103.json"
            )
            with open(os.path.join(path, "group103.json"), "w") as f:
                json.dump(response.json(), f)
        if not os.path.isfile(os.path.join(path, "options.json")):
            response = requests.get(
                "https://raw.githubusercontent.com/makkoncept/doptions/master/options.json"
            )
            with open(os.path.join(path, "options.json"), "w") as f:
                json.dump(response.json(), f)

        if not os.path.isfile(os.path.join(path, "usergroups.json")):
            cprint("Creating necessary config files", color="yellow")
            with open(os.path.join(path, "usergroups.json"), "w") as f:
                json.dump(
                    {1: 0, 2: 0, 3: 0, 4: 0, 5: 0, 6: 0, 7: 0, 8: 0, 9: 0, 10: 0}, f
                )
        if not os.path.isfile(os.path.join(path, "defaultgroups.json")):
            with open(os.path.join(path, "defaultgroups.json"), "w") as f:
                json.dump({101: 800, 102: 800, 103: 800}, f)
        if not os.path.isfile(os.path.join(path, "all_words.json")):
            with open(os.path.join(path, "all_words.json"), "w") as f:
                json.dump({}, f)
        if not os.path.isfile(os.path.join(path, "pyvoc.config")):
            create_config_file()

    except IOError:
        print("Error occured while creating config files.")
        ()


def create_config_file():
    app_id, app_key = get_api_keys()
    cprint(
        "Successfully fetched API key. These API keys are shared among many users. It is recommended that you get your own free personal API keys from https://developer.oxforddictionaries.com/ and add them in '~/.pyvoc/pyvoc.config' file.",
        color="green",
    )

    config["API"] = {"app_id": app_id, "app_key": app_key}
    with open(config_file_path, "w") as f:
        config.write(f)


def read_config_file():
    config.read(config_file_path)
    return config["API"]["app_id"], config["API"]["app_key"]


def get_api_keys():
    url = "http://13.71.3.249/pyvoc"
    cprint(
        "No API key found. Fetching new one...",
        color="yellow",
    )
    response = requests.get(url)
    json_response = response.json()
    if response.status_code != 200 or json_response["message"] == "error":
        cprint("cannot get api key.")
        exit()
    api_id = json_response["app_id"]
    api_key = json_response["app_key"]

    return api_id, api_key



import json
import os

from termcolor import cprint
import colorama

from pyvoc import pyvoc
from pyvoc.check_config import check_config_dir, config_dir_path
from pyvoc.settings import USER_GROUP_ENTRIES_LIMIT

colorama.init()

# create an empty vocabulary group file
def create_new_vocab_group(group_path, group_number):
    pyvoc.stop_loading_animation()
    cprint("creating ", color="cyan", attrs=["bold"], end="")
    cprint("vocabulary group number {}...".format(group_number), color="green")

    with open(group_path, "w") as f:
        json.dump({}, f)


# selects the first user group which has the space for one more word
def select_user_group():
    path = os.path.join(config_dir_path(), "usergroups.json")
    usergroups = dict()
    with open(path, "r") as f:
        usergroups = json.load(f)

    for user_group_number, group_entries in usergroups.items():
        if group_entries < USER_GROUP_ENTRIES_LIMIT:
            return user_group_number

    cprint("No space left for new entries in user groups.", color="red")
    exit()


def check_if_group_full(group_number):
    path = os.path.join(config_dir_path(), "usergroups.json")

    with open(path, "r") as f:
        content = json.load(f)

    if content[str(group_number)] >= USER_GROUP_ENTRIES_LIMIT:
        cprint(
            "cannot add more words to group number {}".format(group_number),
            color="yellow",
        )
        exit()


def counter_increment(group_number):
    path = os.path.join(config_dir_path(), "usergroups.json")

    with open(path, "r") as f:
        content = json.load(f)

    content[str(group_number)] += 1

    with open(path, "w") as f:
        json.dump(content, f)


def add_word_to_vocab(word, parsed_response, group_number=None):
    check_config_dir()
    config_path = config_dir_path()
    definition = {word: parsed_response}

    if not group_number:
        group_number = select_user_group()

    group_path = os.path.join(config_dir_path(), "group" + str(group_number) + ".json")
    if not os.path.isfile(group_path):
        create_new_vocab_group(group_path, group_number)

    check_if_group_full(group_number)

    pyvoc.stop_loading_animation()

    cprint("\nwriting to vocabulary group...", color="yellow")
    with open(group_path, "r") as f:
        content = json.load(f)
    content.update(definition)
    with open(group_path, "w") as f:
        json.dump(content, f, ensure_ascii=False)

    # increase the count of word entries in the group by 1
    counter_increment(group_number)

    # add word (not definition) to all_words.json
    with open(os.path.join(config_path, "all_words.json"), "r") as f:
        all_words = json.load(f)
    all_words.update({word: True})
    with open(os.path.join(config_path, "all_words.json"), "w") as f:
        json.dump(all_words, f)

    cprint("word added to ", color="green", end="")
    cprint("group number {}".format(group_number), color="cyan", attrs=["bold"])


def list_all_groups():
    # reading user groups name and size
    usergroups_path = os.path.join(config_dir_path(), "usergroups.json")
    with open(usergroups_path, "r") as f:
        user_group_numbers = json.load(f)

    default_group_numbers = {101: 800, 102: 800, 103: 800}
    pyvoc.stop_loading_animation()

    # print user groups
    cprint("\nUSER GROUPS", color="cyan", on_color="on_grey")
    cprint("Group no.", color="green", end=" " * (14 - len("Group no")))
    cprint("No. of words")
    for group in user_group_numbers:
        cprint(group, color="green", end=" " * (15 - len(str(group))))
        cprint(str(user_group_numbers[group]))

    # print default groups
    cprint("\nDEFAULT GROUP", color="cyan", on_color="on_grey")
    cprint("Group no.", color="green", end=" " * (14 - len("Group no")))
    cprint("No. of words")
    for group in default_group_numbers:
        cprint(group, color="green", end=" " * (15 - len(str(group))))
        cprint(str(default_group_numbers[group]))
    exit()
